#include <basic.h> // this is a custom library
#include <Wire.h>

//#define debug

byte x; // axis data
byte y;
byte z;
byte l; // left motor speed 
byte r; // right "     "
boolean butC; // buttons
boolean butZ;
byte nunchuckBuffer[6]; // buffer for raw nunchuck data

void setup()
{
  analogOff(); // turn off analog inputs so I2C will work
  nunchuckIni(); // start I2C
  Serial.begin(2400); // start radio
  pinMode(3, OUTPUT);
}

void sendData(byte data, byte dataType_) // this function sends data through radio
{
  //for(byte i = 0; i < 1; i++) // resend counter, not needed
  //{
  Serial.print(dataType_, BYTE);
  Serial.print(data, BYTE);
  //}
}

void mix()
{
  // calculates desired motor speeds
  int dy; // d means delta
  int dx;
  int dl;
  int dr;
  int y_ = y;
  int x_ = x;
  int l_ = l;
  int r_ = r;

  if(y_ >= 128) // if forward
  {
    dy = y_ - 128;
    if(x_ <= 128) // if left
    {
      dx = 128 - x_;
      dl = (dy / 2) - (dx / 2);
      dr = (dy / 2) + (dx / 2);
    }
    else if(x_ > 128) // if right
    {
      dx = x_ - 128;
      dl = (dy / 2) + (dx / 2);
      dr = (dy / 2) - (dx / 2);
    }
    l_ = 128 + dl * 2;
    r_ = 128 + dr * 2;
  }
  else if(y_ < 128) // if reverse
  {
    dy = 128 - y_;
    if(x_ <= 128) // if left
    {
      dx = 128 - x_;
      dr = (dy / 2) - (dx / 2);
      dl = (dy / 2) + (dx / 2);
    }
    else if(x_ > 128) // if right
    {
      dx = x_ - 128;
      dr = (dy / 2) + (dx / 2);
      dl = (dy / 2) - (dx / 2);
    }
    l_ = 128 - dl * 2;
    r_ = 128 - dr * 2;
  }
  l = constrain(l_, 0, 255); // make sure it is in range
  r = constrain(r_, 0, 255);
}

void loop()
{
  byte dataType; // data type to be sent
  byte packet; // actual data prepared to be sent
  byte temp;
  while(1) // loop forever
  {
    nunchuckRead(); // read data from nunchucks

    // the purpose of true range is to make the number from 0 to 255, not 20 to 235 or whatever
    temp = nunchuckBuffer[0];
    nunchuckBuffer[0] = trueRange(temp, 128 - 100 + 15, 128 + 100 + 10 - 15);
    temp = nunchuckBuffer[1];
    nunchuckBuffer[1] = trueRange(temp, 128 - 100 + 15, 128 + 100 + 10 - 15);
    temp = nunchuckBuffer[2];
    nunchuckBuffer[2] = trueRange(temp, 71, 175);
    temp = nunchuckBuffer[3];
    nunchuckBuffer[3] = trueRange(temp, 71, 175);

    //temp = nunchuckBuffer[4];
    //nunchuckBuffer[4] = trueRange(temp, 0x4A, 0xB1);

    if((nunchuckBuffer[5] & B00000001) == 0) // find Z button
    {
      butZ = true; // now use accelerometer data as axis data
      x = nunchuckBuffer[2];
      y = nunchuckBuffer[3];
    } 
    else {
      butZ = false; // use joystick daa as axis data
      x = nunchuckBuffer[0];
      y = nunchuckBuffer[1];
    }
    if((nunchuckBuffer[5] & B00000010) == 0)
    {
      butC = true; // use joystick data
      x = nunchuckBuffer[0];
      y = nunchuckBuffer[1];
    } 
    else {
      butC = false;
    }

    z = nunchuckBuffer[4];
    
    // this makes the joystick tolerant of slight misalignments
    if((x > 128 - 10) & (x < 128 + 10))
    {
      x = 128;
    }
    if((y > 128 - 10) & (y < 128 + 10))
    {
      y = 128;
    }
    
    mix(); // calculate motor speeds
    
    if(butC)
    {
      // button C is used to control head and neck servos
      dataType = B11100111; // this is the dataType for head control
      packet = (x & 0xF0) + ((y & 0xF0) >> 4); // prepare packet
    } 
    else {
      dataType = B11011011; // this is the dataType for drive control
      packet = (l & 0xF0) + ((r & 0xF0) >> 4);
    }
    if((packet == B11100111) || (packet == B11011011)) // make sure data is not conflicting
    {
      packet = packet + 1;
    }
#ifndef debug
    sendData(packet, dataType); // send data to radio
#endif
    analogWrite(3, y);
#ifdef debug // if you turn on debug, use the serial monitor to read all data here
    Serial.print("X: ");
    Serial.println(x ,DEC);
    Serial.print("Y: ");
    Serial.println(y ,DEC);
    Serial.print("Z - Axis Acceleration: ");
    Serial.println(z ,DEC);
    Serial.print("Button Z: ");
    Serial.println(butZ ,DEC);
    Serial.print("Button C: ");
    Serial.println(butC ,DEC);
    Serial.print("Left: ");
    Serial.println(l ,DEC);
    Serial.print("Right: ");
    Serial.println(r ,DEC);
    Serial.print("Data Type: ");
    Serial.println(dataType, BIN);
    Serial.print("Packet: ");
    Serial.println(packet, HEX);
    delay(100);
#endif
  }
}
